@@ -6,30 +6,33 @@ use Carp;
 use strict;
 #use diagnostics;
 #use warnings;
-use NetAddr::IP::Util qw(
+use NetAddr::IP::InetBase qw(
 	inet_any2n
+	isIPv4
+	inet_n2dx
+	inet_aton
+	ipv6_aton
+	ipv6_n2x
+	fillIPv4
+);	
+use NetAddr::IP::Util qw(
 	addconst
 	sub128
 	ipv6to4
 	notcontiguous
-	isIPv4
 	shiftleft
-	inet_n2dx
 	hasbits
 	bin2bcd
 	bcd2bin
-	inet_aton
-	inet_any2n
-	ipv6_aton
-	ipv6_n2x
 	mask4to6
 	ipv4to6
 	naip_gethostbyname
+	havegethostbyname2
 );
 
 use vars qw(@ISA @EXPORT_OK $VERSION $Accept_Binary_IP $Old_nth $AUTOLOAD *Zero);
 
-$VERSION = do { my @r = (q$Revision: 1.24 $ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r };
+$VERSION = do { my @r = (q$Revision: 1.51 $ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r };
 
 require Exporter;
 
@@ -45,6 +48,10 @@ $Accept_Binary_IP = 0;
 $Old_nth = 0;
 *Zero = \&Zeros;
 
+=pod
+
+=encoding UTF-8
+
 =head1 NAME
 
 NetAddr::IP::Lite - Manages IPv4 and IPv6 addresses and subnets
@@ -63,6 +70,8 @@ NetAddr::IP::Lite - Manages IPv4 and IPv6 addresses and subnets
   );
 
   my $ip = new NetAddr::IP::Lite '127.0.0.1';
+	or if your prefer
+  my $ip = NetAddr::IP::Lite->new('127.0.0.1);
 	or from a packed IPv4 address
   my $ip = new_from_aton NetAddr::IP::Lite (inet_aton('127.0.0.1'));
 	or from an octal filtered IPv4 address
@@ -107,8 +116,8 @@ reason, then type:
 
 This module provides an object-oriented abstraction on top of IP
 addresses or IP subnets, that allows for easy manipulations. Most of the
-operations of NetAddr::IP are supported. This module will work older
-versions of Perl and does B<not> use Math::BigInt.
+operations of NetAddr::IP are supported. This module will work with older
+versions of Perl and is compatible with Math::BigInt.
 
 * By default B<NetAddr::IP> functions and methods return string IPv6
 addresses in uppercase.  To change that to lowercase:
@@ -190,6 +199,7 @@ my $_zero = pack('L4',0,0,0,0);
 my $_ones = ~$_zero;
 my $_v4mask = pack('L4',0xffffffff,0xffffffff,0xffffffff,0);
 my $_v4net = ~ $_v4mask;
+my $_ipv4FFFF = pack('N4',0,0,0xffff,0);
 
 sub Zeros() {
   return $_zero;
@@ -352,7 +362,7 @@ Add a 32 bit signed constant to the address part of a NetAddr object.
 This operation changes the address part to point so many hosts above the
 current objects start address. For instance, this code:
 
-    print NetAddr::IP::Lite->new('127.0.0.1') + 5;
+    print NetAddr::IP::Lite->new('127.0.0.1/8') + 5;
 
 will output 127.0.0.6/8. The address will wrap around at the broadcast
 back to the network address. This code:
@@ -499,6 +509,8 @@ sub _new ($$$) {
 
 =item C<-E<gt>new6([$addr, [ $mask]])>
 
+=item C<-E<gt>new6FFFF([$addr, [ $mask]])>
+
 =item C<-E<gt>new_no([$addr, [ $mask]])>
 
 =item C<-E<gt>new_from_aton($netaddr)>
@@ -509,9 +521,14 @@ sub _new ($$$) {
 
 =item C<-E<gt>new_cis6("$addr $mask)>
 
-The first two methods create a new address with the supplied address in
+The first three methods create a new address with the supplied address in
 C<$addr> and an optional netmask C<$mask>, which can be omitted to get 
-a /32 or /128 netmask for IPv4 / IPv6 addresses respectively.
+a /32 or /128 netmask for IPv4 / IPv6 addresses respectively. 
+
+new6FFFF specifically returns an IPv4 address in IPv6 format according to RFC4291
+
+  new6		     ::xxxx:xxxx
+  new6FFFF	::FFFF:xxxx:xxxx
 
 The third method C<new_no> is exclusively for IPv4 addresses and filters
 improperly formatted
@@ -569,6 +586,8 @@ To accept addresses in that format, invoke the module as in
 
 If called with no arguments, 'default' is assumed.
 
+If called with an empty string as the argument, returns 'undef'
+
 C<$addr> can be any of the following and possibly more...
 
   n.n
@@ -602,24 +621,34 @@ Any RFC1884 notation
   ::x:x/host
   0xABCDEF, 0b111111000101011110 within the limits
   of perl's number resolution
-  123456789012  a 'big' bcd number i.e. Math::BigInt
+  123456789012  a 'big' bcd number (bigger than perl likes)
+  and Math::BigInt
 
 If called with no arguments, 'default' is assumed.
 
+If called with and empty string as the argument, 'undef' is returned;
+
 =cut
 
+my $lbmask = inet_aton('255.0.0.0');
+my $_p4broad	= inet_any2n('255.255.255.255');
+my $_p4loop	= inet_any2n('127.0.0.1');
+my $_p4mloop	= inet_aton('255.0.0.0');
+   $_p4mloop	= mask4to6($_p4mloop);
+my $_p6loop	= inet_any2n('::1');
+
 my %fip4 = (
         default         => Zeros,
         any             => Zeros,
-        broadcast       => inet_any2n('255.255.255.255'),
-        loopback        => inet_any2n('127.0.0.1'),
+        broadcast       => $_p4broad,
+        loopback        => $_p4loop,
 	unspecified	=> undef,
 );
 my %fip4m = (
         default         => Zeros,
         any             => Zeros,
         broadcast       => Ones,
-        loopback        => mask4to6(inet_aton('255.0.0.0')),
+        loopback        => $_p4mloop,
 	unspecified	=> undef,	# not applicable for ipV4
 	host		=> Ones,
 );
@@ -628,7 +657,7 @@ my %fip6 = (
 	default         => Zeros,
 	any             => Zeros,
 	broadcast       => undef,	# not applicable for ipV6
-	loopback        => inet_any2n('::1'),
+	loopback        => $_p6loop,
 	unspecified     => Zeros,
 );
 
@@ -682,6 +711,12 @@ sub new6($;$$) {
   goto &_xnew;
 }
 
+sub new6FFFF($;$$) {
+  my $ip = _xnew(1,@_);
+  $ip->{addr} |= $_ipv4FFFF;
+  return $ip;
+}
+
 sub new_cis($;$$) {
   my @in = @_;
   if ( $in[1] && $in[1] =~ m!^(.+)\s+(.+)$! ) {
@@ -714,18 +749,35 @@ sub _xnew($$;$$) {
   }
   my $proto	= shift;
   my $class	= ref $proto || $proto || __PACKAGE__;
-  my $ip	= lc shift;
+  my $ip	= shift;
+
+# fix for bug #75976
+  return undef if defined $ip && $ip eq '';
+
   $ip = 'default' unless defined $ip;
+  $ip = _retMBIstring($ip)		# treat as big bcd string
+	if ref $ip && ref $ip eq 'Math::BigInt';	# can /CIDR notation
   my $hasmask = 1;
   my($mask,$tmp);
 
+# IP to lower case AFTER ref test for Math::BigInt. 'lc' strips blessing
+
+  $ip = lc $ip;
+
   while (1) {
+# process IP's with no CIDR or that have the CIDR as part of the IP argument string
     unless (@_) {
 #      if ($ip =~ m!^(.+)/(.+)$!) {
-      if ($ip =~ m!^([a-z0-9.:-]+)(?:/|\s+)([a-z0-9.:-]+)$!) {
+      if ($ip !~ /\D/) {		# binary number notation
+	$ip = bcd2bin($ip);
+	$mask = Ones;
+	last;
+      }
+      elsif ($ip =~ m!^([a-z0-9.:-]+)(?:/|\s+)([a-z0-9.:-]+)$! ||
+	     $ip =~ m!^[\[]{1}([a-z0-9.:-]+)(?:/|\s+)([a-z0-9.:-]+)[\]]{1}$!) {
 	$ip	= $1;
 	$mask	= $2;
-      } elsif (grep($ip eq $_,qw(default any broadcast loopback unspecified))) {
+      } elsif (grep($ip eq $_,(qw(default any broadcast loopback unspecified)))) {
 	$isV6 = 1 if $ip eq 'unspecified';
 	if ($isV6) {
 	  $mask = $fip6m{$ip};
@@ -737,49 +789,108 @@ sub _xnew($$;$$) {
 	last;
       }
     }
+# process "ipv6" token and default IP's
     elsif (defined $_[0]) {
       if ($_[0] =~ /ipv6/i || $isV6) {
-	if (grep($ip eq $_,qw(default any loopback unspecified))) {
+	if (grep($ip eq $_,(qw(default any loopback unspecified)))) {
 	  $mask	= $fip6m{$ip};
 	  $ip	= $fip6{$ip};
 	  last;
 	} else {
 	  return undef unless $isV6;
+# add for ipv6 notation "12345, 1"
         }
-      } else {
-	$mask = lc $_[0];
+#	$mask = lc $_[0];
+#      } else {
+#	$mask = lc $_[0];
       }
+# extract mask
+      $mask = $_[0];
     }
+###
+### process mask
     unless (defined $mask) {
       $hasmask	= 0;
       $mask	= 'host';
     }
 
-# parse mask
-    if ($mask =~ /^(\d+)$/) {
-      my $mval = $1;
-      if (! $isV6 && index($ip,':') < 0) {	# is ipV4
-	if ($mval == 32) {				# cidr 32
+# two kinds of IP's can turn on the isV6 flag
+# 1) big digits that are over the IPv4 boundry
+# 2) IPv6 IP syntax
+#
+# check these conditions and set isV6 as appropriate
+#
+    my $try;
+    $isV6 = 1 if	# check big bcd and IPv6 rfc1884
+	( $ip !~ /\D/ && 				  # ip is all decimal
+	  (length($ip) > 3 || $ip > 255) &&		  # exclude a single digit in the range of zero to 255, could be funny IPv4
+	  ($try = bcd2bin($ip)) && ! isIPv4($try)) ||	  # precedence so $try is not corrupted
+	(index($ip,':') >= 0 && ($try = ipv6_aton($ip))); # fails if not an rfc1884 address
+
+# if either of the above conditions is true, $try contains the NetAddr 128 bit address
+
+# checkfor Math::BigInt mask
+    $mask = _retMBIstring($mask)				# treat as big bcd string
+        if ref $mask && ref $mask eq 'Math::BigInt';
+
+# MASK to lower case AFTER ref test for Math::BigInt, 'lc' strips blessing
+
+    $mask = lc $mask;
+
+    if ($mask !~ /\D/) {				# bcd or CIDR notation
+      my $isCIDR = length($mask) < 4 && $mask < 129;
+      if ($isV6) {
+	if ($isCIDR) {
+	  my($dq1,$dq2,$dq3,$dq4);
+	  if ($ip =~ /^(\d+)(?:|\.(\d+)(?:|\.(\d+)(?:|\.(\d+))))$/ &&
+	    do {$dq1 = $1;
+		$dq2 = $2 || 0;
+		$dq3 = $3 || 0;
+		$dq4 = $4 || 0;
+		1;
+	    } &&
+	    $dq1 >= 0 && $dq1 < 256 &&
+	    $dq2 >= 0 && $dq2 < 256 &&
+	    $dq3 >= 0 && $dq3 < 256 &&
+	    $dq4 >= 0 && $dq4 < 256
+	  ) {	# corner condition of IPv4 with isV6
+	    $ip = join('.',$dq1,$dq2,$dq3,$dq4);
+	    $try = ipv4to6(inet_aton($ip));
+	    if ($mask < 32) {
+	      $mask = shiftleft(Ones,32 -$mask);
+	    }
+	    elsif ($mask == 32) {
+	      $mask = Ones;
+	    } else {
+	      return undef;			# undoubtably an error
+	    }
+	  }
+	  elsif ($mask < 128) {
+	    $mask = shiftleft(Ones,128 -$mask);	# small cidr
+	  } else {
+	    $mask = Ones();
+	  }
+	} else {
+	  $mask = bcd2bin($mask);
+	}
+      }
+      elsif ($isCIDR && $mask < 33) {		# is V4
+	if ($mask < 32) {
+	  $mask = shiftleft(Ones,32 -$mask);
+	}
+	elsif ( $mask == 32) {
 	  $mask = Ones;
+	} else {
+	  $mask = bcd2bin($mask);
+	  $mask |= $_v4mask;			# v4 always 
 	}
-	elsif ($mask < 32) {			# small cidr
-	  $mask = shiftleft(Ones,32 -$mval);
-	} else {				# is a binary mask
-	  $mask = pack('L3N',0xffffffff,0xffffffff,0xffffffff,$mval);
-	}
-      } else {					# is ipV6
-	$isV6	= 1;
-	if ($mval == 128) {			# cidr 128
-	  $mask = Ones;
-	}
-	elsif (index($ip,':') < 0) {	# corner case of ipV4 with new6
-	  $mask = shiftleft(Ones,32 -$mval);
-	}
-	elsif ($mask < 128) {			# small cidr
-	  $mask = shiftleft(Ones,128 -$mval);
-	} else {				# is a binary mask
-	  $mask = bcd2bin($mval);
-	}
+      } else {					# also V4
+	$mask = bcd2bin($mask);
+	$mask |= $_v4mask;
+      }
+      if ($try) {				# is a big number
+	$ip = $try;
+	last;
       }
     } elsif ($mask =~ m/^\d+\.\d+\.\d+\.\d+$/) { # ipv4 form of mask
       $mask = _no_octal($mask) if $noctal;	# filter for octal
@@ -795,13 +906,13 @@ sub _xnew($$;$$) {
       return undef unless defined ($mask = ipv6_aton($mask));	# try ipv6 form of mask
     }
 
-# parse IP
+# process remaining IP's
 
     if (index($ip,':') < 0) {				# ipv4 address
       if ($ip =~ m/^(\d+)\.(\d+)\.(\d+)\.(\d+)$/) {
 	;	# the common case
       }
-      elsif (grep($ip eq $_,qw(default any broadcast loopback))) {
+      elsif (grep($ip eq $_,(qw(default any broadcast loopback)))) {
 	return undef unless defined ($ip = $fip4{$ip});
 	last;
       }
@@ -818,10 +929,12 @@ sub _xnew($$;$$) {
       elsif ($ip =~ /^(\d+)$/ && $hasmask && $1 >= 0 and $1 < 256) { # pure numeric
 	$ip = sprintf("%d.0.0.0",$1);
       }
-      elsif ($ip =~ /^\d+$/ && !$hasmask) {	# a big integer
+#      elsif ($ip =~ /^\d+$/ && !$hasmask) {	# a big integer
+      elsif ($ip =~ /^\d+$/ ) {	# a big integer
 	$ip = bcd2bin($ip);
 	last;
       }
+# these next three might be broken??? but they have been in the code a long time and no one has complained
       elsif ($ip =~ /^0[xb]\d+$/ && $hasmask &&
 		(($tmp = eval "$ip") || 1) &&
 		$tmp >= 0 && $tmp < 256) {
@@ -889,12 +1002,12 @@ sub _xnew($$;$$) {
 	last;
       }
 # check for resolvable IPv4 hosts
-      elsif ($ip !~ /[^a-zA-Z0-9\.-]/ && ($tmp = gethostbyname($ip)) && $tmp ne $_v4zero && $tmp ne $_zero ) {
+      elsif ($ip !~ /[^a-zA-Z0-9\.-]/ && ($tmp = gethostbyname(fillIPv4($ip))) && $tmp ne $_v4zero && $tmp ne $_zero ) {
 	$ip = ipv4to6($tmp);
 	last;
       }
 # check for resolvable IPv6 hosts
-      elsif ($ip !~ /[^a-zA-Z0-9\.-]/ && ($tmp = naip_gethostbyname($ip))) {
+      elsif ($ip !~ /[^a-zA-Z0-9\.-]/ && havegethostbyname2() && ($tmp = naip_gethostbyname($ip))) {
 	$ip = $tmp;
 	$isV6 = 1;
 	last;
@@ -918,16 +1031,16 @@ sub _xnew($$;$$) {
 ########## continuing
     else {						# ipv6 address
       $isV6 = 1;
+      $ip = $1 if $ip =~ /\[([^\]]+)\]/;		# transform URI notation
       if (defined ($tmp = ipv6_aton($ip))) {
 	$ip = $tmp;
 	last;
       }
-      last if grep($ip eq $_,qw(default any loopback unspecified)) &&
+      last if grep($ip eq $_,(qw(default any loopback unspecified))) &&
 		defined ($ip = $fip6{$ip});
       return undef;
     }
   } # end while (1)
-
   return undef if notcontiguous($mask);			# invalid if not contiguous
 
   my $self = {
@@ -1077,7 +1190,7 @@ sub range ($) {
 
 When called in a scalar context, will return a numeric representation
 of the address part of the IP address. When called in an array
-contest, it returns a list of two elements. The first element is as
+context, it returns a list of two elements. The first element is as
 described, the second element is the numeric representation of the
 netmask.
 
@@ -1102,6 +1215,112 @@ sub numeric ($) {
     : bin2bcd($_[0]->{addr});
 }
 
+=item C<-E<gt>bigint()>
+
+When called in a scalar context, will return a Math::BigInt representation
+of the address part of the IP address. When called in an array
+contest, it returns a list of two elements. The first element is as
+described, the second element is the Math::BigInt  representation of the
+netmask.
+
+=cut
+
+my $biloaded;
+my $bi2strng;
+my $no_mbi_emu = 1;
+
+# function to force into test development mode
+#
+sub _force_bi_emu {
+  undef $biloaded;
+  undef $bi2strng;
+  $no_mbi_emu = 0;
+  print STDERR "\n\n\tWARNING: test development mode, this
+\tmessage SHOULD NEVER BE SEEN IN PRODUCTION!
+set my \$no_mbi_emu = 1 in t/bigint.t to remove this warning\n\n";
+}
+
+# function to stringify various flavors of Math::BigInt objects
+# tests to see if the object is a hash or a signed scalar
+
+sub _bi_stfy {
+  "$_[0]" =~ /(\d+)/;		# stringify and remove '+' if present
+  $1;
+}
+
+sub _fakebi2strg {
+  ${$_[0]} =~ /(\d+)/;
+  $1;
+}
+
+# fake new from bi string Math::BigInt 0.01
+#
+sub _bi_fake {
+  bless \('+'. $_[1]), 'Math::BigInt';
+}
+
+# as of this writing there are three known flavors of Math::BigInt
+# v0.01         MBI::new returns a scalar ref
+# v1.?? - 1.69  CALC::_new takes a reference to a scalar, returns an array, MBI returns a hash ref
+# v1.70 and up  CALC::_new takes a scalar, returns and array, MBI returns a hash ref
+
+sub _loadMBI {						# load Math::BigInt on demand
+  if (eval {$no_mbi_emu && require Math::BigInt}) {	# any version should work, three known
+    import Math::BigInt;
+    $biloaded = \&Math::BigInt::new;
+    $bi2strng = \&_bi_stfy;
+  } else {
+    $biloaded = \&_bi_fake;
+    $bi2strng = \&_fakebi2strg;
+  }
+}
+
+sub _retMBIstring {
+  _loadMBI unless $biloaded;				# load Math::BigInt on demand
+  $bi2strng->(@_);
+}
+
+sub _biRef {
+  _loadMBI unless $biloaded;				# load Math::BigInt on demand
+  $biloaded->('Math::BigInt',$_[0]);
+}
+
+sub bigint($) {
+  my($addr,$mask);
+  if (wantarray) {
+    if (! $_[0]->{isv6} && isIPv4($_[0]->{addr})) {
+      $addr = $_[0]->{addr}
+	? sprintf("%u",unpack('N',ipv6to4($_[0]->{addr})))
+	: 0;
+      $mask = $_[0]->{mask}
+	? sprintf("%u",unpack('N',ipv6to4($_[0]->{mask})))
+	: 0;
+    }
+    else {
+      $addr = $_[0]->{addr}
+	? bin2bcd($_[0]->{addr})
+	: 0;
+      $mask = $_[0]->{mask}
+	? bin2bcd($_[0]->{mask})
+	: 0;
+    }
+    (_biRef($addr),_biRef($mask));
+
+  } else {	# not wantarray
+
+    if (! $_[0]->{isv6} && isIPv4($_[0]->{addr})) {
+      $addr = $_[0]->{addr}
+	? sprintf("%u",unpack('N',ipv6to4($_[0]->{addr})))
+	: 0;
+    } else {
+      $addr = $_[0]->{addr}
+	? bin2bcd($_[0]->{addr})
+	: 0;
+    }
+    _biRef($addr);
+  }
+}
+
 =item C<$me-E<gt>contains($other)>
 
 Returns true when C<$me> completely contains C<$other>. False is
@@ -1132,6 +1351,37 @@ sub within ($$) {
 	? 1 : 0;
 }
 
+=item C-E<gt>is_rfc1918()>
+
+Returns true when C<$me> is an RFC 1918 address.
+
+     10.0.0.0        -   10.255.255.255  (10/8 prefix)
+     172.16.0.0      -   172.31.255.255  (172.16/12 prefix)
+     192.168.0.0     -   192.168.255.255 (192.168/16 prefix)
+
+=cut
+
+my $ip_10	= NetAddr::IP::Lite->new('10.0.0.0/8');
+my $ip_10n	= $ip_10->{addr};               # already the right value
+my $ip_10b	= $ip_10n | ~ $ip_10->{mask};
+
+my $ip_172	= NetAddr::IP::Lite->new('172.16.0.0/12');
+my $ip_172n	= $ip_172->{addr};              # already the right value
+my $ip_172b	= $ip_172n | ~ $ip_172->{mask};
+
+my $ip_192	= NetAddr::IP::Lite->new('192.168.0.0/16');
+my $ip_192n	= $ip_192->{addr};              # already the right value
+my $ip_192b	= $ip_192n | ~ $ip_192->{mask};
+
+sub is_rfc1918 ($) {
+  my $netme     = $_[0]->{addr} & $_[0]->{mask};
+  my $brdme     = $_[0]->{addr} | ~ $_[0]->{mask};
+  return 1 if (sub128($netme,$ip_10n) && sub128($ip_10b,$brdme));
+  return 1 if (sub128($netme,$ip_192n) && sub128($ip_192b,$brdme));
+  return (sub128($netme,$ip_172n) && sub128($ip_172b,$brdme))
+        ? 1 : 0;
+}
+
 =item C<-E<gt>first()>
 
 Returns a new object representing the first usable IP address within
@@ -1139,8 +1389,15 @@ the subnet (ie, the first host address).
 
 =cut
 
+my $_cidr127 = pack('N4',0xffffffff,0xffffffff,0xffffffff,0xfffffffe);
+
 sub first ($) {
-  return $_[0]->network + 1;
+  if (hasbits($_[0]->{mask} ^ $_cidr127)) {
+    return $_[0]->network + 1;
+  } else {
+    return $_[0]->network;
+  }
+#  return $_[0]->network + 1;
 }
 
 =item C<-E<gt>last()>
@@ -1151,7 +1408,12 @@ the subnet (ie, one less than the broadcast address).
 =cut
 
 sub last ($) {
-  return $_[0]->broadcast - 1;
+  if (hasbits($_[0]->{mask} ^ $_cidr127)) {
+    return $_[0]->broadcast - 1;
+  } else {
+    return $_[0]->broadcast;
+  }
+#  return $_[0]->broadcast - 1;
 }
 
 =item C<-E<gt>nth($index)>
@@ -1181,20 +1443,23 @@ To use the old behavior for C<-E<gt>nth($index)> and C<-E<gt>num()>:
   NetAddr::IP->new('10/30')->nth(3) == 10.0.0.3/30
 
 Note that in each case, the broadcast address is represented in the
-output set and that the 'zero'th index is alway undef.
+output set and that the 'zero'th index is alway undef except for
+a point-to-point /31 or /127 network where there are exactly two
+addresses in the network.
 
   new behavior:
   NetAddr::IP->new('10/32')->nth(0)  == 10.0.0.0/32
   NetAddr::IP->new('10.1/32'->nth(0) == 10.0.0.1/32
-  NetAddr::IP->new('10/31')->nth(0)  == undef
-  NetAddr::IP->new('10/31')->nth(1)  == undef
+  NetAddr::IP->new('10/31')->nth(0)  == 10.0.0.0/32
+  NetAddr::IP->new('10/31')->nth(1)  == 10.0.0.1/32
   NetAddr::IP->new('10/30')->nth(0) == 10.0.0.1/30
   NetAddr::IP->new('10/30')->nth(1) == 10.0.0.2/30
   NetAddr::IP->new('10/30')->nth(2) == undef
 
-Note that a /32 net always has 1 usable address while a /31 has none since
-it has a network and broadcast address, but no host addresses. The first
-index (0) returns the address immediately following the network address.
+Note that a /32 net always has 1 usable address while a /31 has exactly 
+two usable addresses for point-to-point addressing. The first
+index (0) returns the address immediately following the network address 
+except for a /31 or /127 when it return the network address.
 
 =cut
 
@@ -1202,39 +1467,87 @@ sub nth ($$) {
   my $self    = shift;
   my $count   = shift;
 
-  ++$count unless ($Old_nth);
-  return undef if ($count < 1 or $count > $self->num ());
+  my $slash31 = ! hasbits($self->{mask} ^ $_cidr127);
+  if ($Old_nth) {
+    return undef if $slash31 && $count != 1;
+    return undef if ($count < 1 or $count > $self->num ());
+  }
+  elsif ($slash31) {
+    return undef if ($count && $count != 1);	# only index 0, 1 allowed for /31
+  } else {
+    ++$count;
+    return undef if ($count < 1 or $count > $self->num ());
+  }
   return $self->network + $count;
 }
 
 =item C<-E<gt>num()>
 
+As of version 4.42 of NetAddr::IP and version 1.27 of NetAddr::IP::Lite
+a /31 and /127 with return a net B<num> value of 2 instead of 0 (zero)
+for point-to-point networks.
+
 Version 4.00 of NetAddr::IP and version 1.00 of NetAddr::IP::Lite
-Returns the number of usable addresses IP addresses within the
-subnet, not counting the broadcast or network address. Previous versions
-returned th number of IP addresses not counting the broadcast address.
+return the number of usable IP addresses within the subnet, 
+not counting the broadcast or network address.
+
+Previous versions worked only for ipV4 addresses, returned a    
+maximum span of 2**32 and returned the number of IP addresses 
+not counting the broadcast address.
+	(one greater than the new behavior)
 
 To use the old behavior for C<-E<gt>nth($index)> and C<-E<gt>num()>:
 
   use NetAddr::IP::Lite qw(:old_nth);
 
+WARNING:
+
+NetAddr::IP will calculate and return a numeric string for network 
+ranges as large as 2**128. These values are TEXT strings and perl
+can treat them as integers for numeric calculations.
+
+Perl on 32 bit platforms only handles integer numbers up to 2**32 
+and on 64 bit platforms to 2**64.
+
+If you wish to manipulate numeric strings returned by NetAddr::IP
+that are larger than 2**32 or 2**64, respectively,  you must load 
+additional modules such as Math::BigInt, bignum or some similar 
+package to do the integer math.
+
 =cut
 
 sub num ($) {
-  my @net = unpack('L3N',$_[0]->{mask} ^ Ones);
   if ($Old_nth) {
+    my @net = unpack('L3N',$_[0]->{mask} ^ Ones);
 # number of ip's less broadcast
     return 0xfffffffe if $net[0] || $net[1] || $net[2]; # 2**32 -1
     return $net[3] if $net[3];
-  } else {	# returns 1 for /32 /128, 0 for /31 /127 else n-2 up to 2**32
-# number of usable IP's === number of ip's less broadcast & network addys
-    return 0xfffffffd if $net[0] || $net[1] || $net[2]; # 2**32 -2
-    return 1 unless $net[3];
-    $net[3]--;
+  } else {	# returns 1 for /32 /128, 2 for /31 /127 else n-2 up to 2**32
+    (undef, my $net) = addconst($_[0]->{mask},1);
+    return 1 unless hasbits($net);	# ipV4/32 or ipV6/128
+    $net = $net ^ Ones;
+    return 2 unless hasbits($net);	# ipV4/31 or ipV6/127
+    $net &= $_v4net unless $_[0]->{isv6};
+    return bin2bcd($net);
   }
-  return $net[3];
 }
 
+# deprecated
+#sub num ($) {
+#  my @net = unpack('L3N',$_[0]->{mask} ^ Ones);
+#  if ($Old_nth) {
+## number of ip's less broadcast
+#    return 0xfffffffe if $net[0] || $net[1] || $net[2]; # 2**32 -1
+#    return $net[3] if $net[3];
+#  } else {	# returns 1 for /32 /128, 0 for /31 /127 else n-2 up to 2**32
+## number of usable IP's === number of ip's less broadcast & network addys
+#    return 0xfffffffd if $net[0] || $net[1] || $net[2]; # 2**32 -2
+#    return 1 unless $net[3];
+#    $net[3]--;
+#  }
+#  return $net[3];
+#}
+
 =pod
 
 =back
@@ -1274,9 +1587,9 @@ sub import {
 	:upper
 	:lower
 
-=head1 AUTHOR
+=head1 AUTHORS
 
-Luis E. Muoz E<lt>luismunoz@cpan.orgE<gt>,
+Luis E. Muñoz E<lt>luismunoz@cpan.orgE<gt>,
 Michael Robinton E<lt>michael@bizsystems.comE<gt>
 
 =head1 WARRANTY
@@ -1284,18 +1597,45 @@ Michael Robinton E<lt>michael@bizsystems.comE<gt>
 This software comes with the  same warranty as perl itself (ie, none),
 so by using it you accept any and all the liability.
 
-=head1 LICENSE
+=head1 COPYRIGHT
 
- This software is (c) Luis E. Muoz, 1999 - 2005
- and (c) Michael Robinton, 2006 - 2010.
+ This software is (c) Luis E. Muñoz, 1999 - 2005
+ and (c) Michael Robinton, 2006 - 2012.
 
-It can be used under the terms of the perl artistic license provided that
-proper credit for the work of the author is preserved in the form of this
-copyright notice and license for this module.
+All rights reserved.
+
+This program is free software; you can redistribute it and/or modify
+it under the terms of either:
+
+  a) the GNU General Public License as published by the Free
+  Software Foundation; either version 2, or (at your option) any
+  later version, or
+
+  b) the "Artistic License" which comes with this distribution.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See either
+the GNU General Public License or the Artistic License for more details.
+
+You should have received a copy of the Artistic License with this
+distribution, in the file named "Artistic".  If not, I'll be glad to provide
+one.
+
+You should also have received a copy of the GNU General Public License
+along with this program in the file named "Copying". If not, write to the
+
+        Free Software Foundation, Inc.,
+        51 Franklin Street, Fifth Floor
+        Boston, MA 02110-1301 USA
+
+or visit their web page on the internet at:
+
+        http://www.gnu.org/copyleft/gpl.html.
 
 =head1 SEE ALSO
 
-perl(1), NetAddr::IP(3), NetAddr::IP::Util(3)
+NetAddr::IP(3), NetAddr::IP::Util(3), NetAddr::IP::InetBase(3)
 
 =cut
 
